# Title:	MT-LJPost
# Summary:	A plugin for posting to livejournal whenever you post/edit an entry to MOveable Type
# Author:	Gavin Mogan (halkeye@halkeye.net) (http://www.halkeye.net/)
# Version:	1.0.4
# Date:		January 30, 2004
#
# History
# 1.0.0 - January 29, 2004
#  - Inital Release
# 1.0.1 - January 30, 2004
#  - Removing needing so many variables
#  - Adding selecting of which blog you want.
#
# 1.0.2 - January 30, 2004
#  - Added config file because I got tired of always having to change the file for release
#      this file goes into your mt directory, not plugins (for now, i'll fix it next version or something)
#  # Format: blogid site username password
#  #         3 www.livejournal.com test test
#  #         4 www.deadjournal.com:90 test test
#  - Added handler to check for LJ::Simple and give a message without it
#
# 1.0.3 - January 30, 2004
#  - Entries will now edit after the inital post.
#  - If an entry doesn't save to livejournal, just hit save again
#  - Added support for exerpts / lj-cut
#  - Date of LJ post is date of Actual Entry. You'll have to watch out for backdated entries
#      If its a problem, set the date to now, post, then reset to old.
#      Temp Fix
#  - Convert Line Breaks actually working.
#
# 1.0.4 - January 20, 2004
#  - Date fix, oooopsies
#
# Information about this plugin can be found at
# http://www.halkeye.net/archives/000073.html (perm link later)
#
# Requires the LJ::Simple perl module - http://www.bpfh.net/computing/software/LJ%3a%3aSimple/
# Or CPAN as i prefer.
#
# Copyright 2004 Gavin Mogan
# This code cannot be redistributed without
# permission from the author. 

my %BLOGS = (); # Which blogs to actually care about

use lib '../lib';
package plugins::MTLJPost;

use warnings;
use strict;
use MT::Entry;
use MT::Blog;
use MT::PluginData;

my $VERSION = '1.0.4';
my $mt_entry_save_handler;

{
	eval {require LJ::Simple};
	die "Need LJ::Simple" if $@;
	eval {require POSIX};
	die "Need POSIX" if $@;

	# Trap ugly redefinition warnings
	local $SIG{__WARN__} = sub {  }; 

	# Replace MT::Entry::save with our 
	# own subroutine commentPost() but save
	# the code in $mt_comments_post_handler.
	#
	$mt_entry_save_handler = \&MT::Entry::save;
	*MT::Entry::save = \&entrySave; 


	if (open(LOG,"MTLJPost.cfg"))
	{
	 	my @lines = <LOG>;
		close(LOG);

		foreach my $line (@lines) {
			chop $line;
			my @args = split / /, $line;
			# 0 - Blogid
			# 1 - Server
			# 2 - username
			# 3 - Password
			my $blog = {
				'site' => $args[1],
				'user' => $args[2],
				'pass' => $args[3],
			};
			$BLOGS{$args[0]} = $blog;
		}
	}
}  
use LJ::Simple;
use POSIX qw(mktime difftime);

sub entrySave {
	my $entry = shift;
	my $err = &$mt_entry_save_handler($entry);
	return $err unless ($err);
	
	# disable to debug
	return $err unless (defined($BLOGS{$entry->blog_id}));
	
	my $blog = MT::Blog->load($entry->blog_id);
	my $data = MT::PluginData->load({ plugin => 'MTLJPost',
	                                  key    => $entry->blog_id . "_" . $entry->id }) || undef;
	# Build up the subject
	my $subject = $blog->name . ": " . $entry->title;
			
	my $body = MT->apply_text_filters($entry->text, $entry->text_filters);
	unless ( defined($entry->text_more) && $entry->text_more =~ /^\s*$/s ) {
		$body .= "<lj-cut>\n";
		$body .= MT->apply_text_filters($entry->text_more, $entry->text_filters);
		$body .= "</lj-cut>";
	}
	$body .= "<br /><br />";
	$body .= "[<a href=\"" . $blog->site_url . "\">" . $blog->name . "</a>] (<a href=\"" . $entry->permalink . "\">Permanent link to this entry</a>)<br />";
	my $date = $entry->created_on;
	$date =~ /(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/;
	my $time_t = mktime($6, $5, $4, $3, $2-1, $1-1900) || die "error";
	my $lj = new LJ::Simple ({
		user 	=> $BLOGS{$entry->blog_id}{'user'},
		pass	=> $BLOGS{$entry->blog_id}{'pass'},
		site	=> $BLOGS{$entry->blog_id}{'site'},
	});
	
	if (!(defined $lj)) {
		#return $entry->error("Failed to log into LiveJournal: $LJ::Simple::error\n\nEntry not posted to lj :(");
		return 0;
	}

	my $event=();
	
	# yes this is
	if (defined $data)
	{
		my %Entries=();
		my $itemid = $data->data;
		$itemid = $$itemid;
		(defined $lj->GetEntries(\%Entries, undef, "one", $itemid )) || die "Failed to get Previous Entry: $LJ::Simple::error\n";
		$event = (values %Entries)[0];
		$lj->SetEntry($event,$body) || die "Failed to set entry: $LJ::Simple::error\n";
		$lj->SetSubject($event,$subject) || die "Failed to set subject: $LJ::Simple::error\n";
#		$lj->Setprop_preformatted($event, $entry->convert_breaks);
		$lj->Setprop_preformatted($event, 1) if ($entry->convert_breaks);
		$lj->Setprop_preformatted($event, 0) unless ($entry->convert_breaks);
		$lj->SetDate($event,$time_t) || die "Failed to set date: $LJ::Simple::error\n";
		$lj->EditEntry($event) || die "$0: Failed to edit entry - $LJ::Simple::error\n";
		
	}
	else
	{
		my %Event=();
		$lj->NewEntry(\%Event) || die "Failed to create new entry: $LJ::Simple::error\n";
		$lj->SetEntry(\%Event,$body) || die "Failed to set entry: $LJ::Simple::error\n";
		$lj->SetSubject(\%Event,$subject) || die "Failed to set subject: $LJ::Simple::error\n";
		$lj->Setprop_preformatted(\%Event, 1) if ($entry->convert_breaks);
		$lj->Setprop_preformatted(\%Event, 0) unless ($entry->convert_breaks);
		$lj->SetDate(\%Event,$time_t) || die "Failed to set date: $LJ::Simple::error\n";
		$lj->Setprop_backdate(\%Event, 1);
		# 3600 Seconds gives you an hour window to post it properly, otherwise it gets marked as back dated and you have to manually fix it.
		$lj->Setprop_backdate(\%Event, 0) if (difftime(time(), $time_t) < 3600);
		$data = MT::PluginData->new;
		$data->plugin('MTLJPost');
		$data->key($entry->blog_id . "_" . $entry->id);

		my ($item_id,$anum,$html_id)=$lj->PostEntry(\%Event);
		(defined $item_id) || die "$0: Failed to post journal entry: $LJ::Simple::error\n";
		$data->data(\$item_id);
		$data->save or die $data->errstr;
	} 
  
} 
1;
